#include <iostream>
#include <vector>
#include <queue>
#include <string>

struct State {
    int mask;
    int steps;
};

State GetSafeShifted(State state, int n, int m, int x, int y, int dx, int dy) {
    if (x + dx < 0 || x + dx >= n || y + dy < 0 || y + dy >= m) {
        return state;  // border cases - can just return same state, it won't get processed twice
    }
    // code below swaps two bits in position (x, y) and (x + dx, y + dy)
    unsigned new_mask = state.mask;
    unsigned pos_one = m * x + y;
    unsigned pos_two = m * (x + dx) + (y + dy);
    unsigned was_one = (new_mask >> pos_one) & 1;
    unsigned was_two = (new_mask >> pos_two) & 1;
    new_mask = new_mask & ~(1 << pos_one);
    new_mask = new_mask & ~(1 << pos_two);
    new_mask = new_mask | (was_one << pos_two);
    new_mask = new_mask | (was_two << pos_one);
    return {(int)new_mask, state.steps + 1};
}

int ReadMask(int n, int m) {
    int res = 0;
    for (int i = 0; i < n; i++) {
        std::string s;
        std::cin >> s;
        for (int j = 0; j < m; j++) {
            res = (res << 1) | (s[j] == '#' ? 1 : 0);
        }
    }
    return res;
}

int main() {
    int n, m, k;
    std::cin >> n >> m >> k;
    int start_mask = ReadMask(n, m);
    int end_mask = ReadMask(n, m);
    std::vector<bool> used(1 << (n * m));
    std::vector<int> dist(1 << (n * m), -1);
    std::vector<std::pair<int, int>> delta = {
        { 1, 0 },
        { 0, 1 },
        { -1, 0 },
        { 0, -1 }
    };
    std::queue<State> que;
    dist[start_mask] = 0;
    used[start_mask] = true;
    que.push({start_mask, 0});
    while (!que.empty()) {
        auto el = que.front();
        que.pop();
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < m; j++) {
                for (auto my_pair: delta) {
                    auto dx = my_pair.first, dy = my_pair.second;
                    auto new_state = GetSafeShifted(el, n, m, i, j, dx, dy);
                    if (!used[new_state.mask]) {
                        que.push(new_state);
                        used[new_state.mask] = true;
                        dist[new_state.mask] = new_state.steps;
                    }
                }
            }
        }
    }
    if (!used[end_mask] || dist[end_mask] > k) {
        std::cout << "IMPOSSIBLE!" << std::endl;
    } else {
        std::cout << dist[end_mask] << std::endl;
    }
}
